Package: SpotClean
Version: 1.14.0
Date: 2024/06/08
Title: SpotClean adjusts for spot swapping in spatial transcriptomics
        data
Authors@R: c(
    person("Zijian", "Ni", email="zni25@wisc.edu", role = c("aut", "cre"), 
        comment = c(ORCID = "0000-0003-1181-8337")),
    person("Christina", "Kendziorski", role = "ctb")
  )
Depends: R (>= 4.2.0),
Imports: stats, methods, utils, dplyr, S4Vectors, SummarizedExperiment,
        SpatialExperiment, Matrix, rhdf5, ggplot2, grid, readbitmap,
        rjson, tibble, viridis, grDevices, RColorBrewer, Seurat, rlang
Suggests: testthat (>= 2.1.0), knitr, BiocStyle, rmarkdown, R.utils,
        spelling
biocViews: DataImport, RNASeq, Sequencing, GeneExpression, Spatial,
        SingleCell, Transcriptomics, Preprocessing
Description: 
    SpotClean is a computational method to adjust for spot swapping in spatial 
    transcriptomics data. Recent spatial transcriptomics experiments utilize 
    slides containing thousands of spots with spot-specific barcodes that bind 
    mRNA. Ideally, unique molecular identifiers at a spot measure spot-specific 
    expression, but this is often not the case due to bleed from nearby spots, 
    an artifact we refer to as spot swapping. SpotClean is able to estimate the 
    contamination rate in observed data and decontaminate the spot swapping 
    effect, thus increase the sensitivity and precision of downstream analyses. 
License: GPL-3
NeedsCompilation: yes
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://github.com/zijianni/SpotClean
BugReports: https://github.com/zijianni/SpotClean/issues
Language: en-US
git_url: https://git.bioconductor.org/packages/SpotClean
git_branch: RELEASE_3_23
git_last_commit: fbf937f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Packaged: 2026-04-29 01:01:29 UTC; biocbuild
Author: Zijian Ni [aut, cre] (ORCID: <https://orcid.org/0000-0003-1181-8337>),
  Christina Kendziorski [ctb]
Maintainer: Zijian Ni <zni25@wisc.edu>
