# Generated by roxygen2: do not edit by hand

S3method(arcScore,SummarizedExperiment)
S3method(arcScore,default)
S3method(spotclean,SpatialExperiment)
S3method(spotclean,SummarizedExperiment)
S3method(visualizeHeatmap,SummarizedExperiment)
S3method(visualizeHeatmap,default)
S3method(visualizeLabel,SummarizedExperiment)
S3method(visualizeLabel,default)
export(arcScore)
export(convertToSeurat)
export(createSlide)
export(keepHighGene)
export(read10xRaw)
export(read10xRawH5)
export(read10xSlide)
export(spotclean)
export(visualizeHeatmap)
export(visualizeLabel)
export(visualizeSlide)
import(Matrix)
import(grid)
importClassesFrom(Matrix,dgCMatrix)
importFrom(Matrix,colSums)
importFrom(Matrix,readMM)
importFrom(Matrix,rowMeans)
importFrom(Matrix,sparseMatrix)
importFrom(RColorBrewer,brewer.pal)
importFrom(S4Vectors,metadata)
importFrom(Seurat,"DefaultAssay<-")
importFrom(Seurat,Cells)
importFrom(Seurat,CreateSeuratObject)
importFrom(Seurat,FindVariableFeatures)
importFrom(Seurat,GetTissueCoordinates)
importFrom(Seurat,NormalizeData)
importFrom(Seurat,Read10X_Image)
importFrom(SpatialExperiment,scaleFactors)
importFrom(SpatialExperiment,spatialCoords)
importFrom(SummarizedExperiment,"assays<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(dplyr,filter)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(ggplot2,Geom)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,ggproto_parent)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_set)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(grDevices,colorRampPalette)
importFrom(methods,as)
importFrom(readbitmap,read.bitmap)
importFrom(rhdf5,h5ls)
importFrom(rhdf5,h5read)
importFrom(rjson,fromJSON)
importFrom(rlang,.data)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,dist)
importFrom(stats,lm)
importFrom(stats,optim)
importFrom(stats,quantile)
importFrom(tibble,tibble)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(viridis,scale_fill_viridis)
importMethodsFrom(S4Vectors,"metadata<-")
importMethodsFrom(S4Vectors,metadata)
